@@ -1,3 +1,20 @@
+/*  Copyright 1996-1999,2001-2003,2007-2009,2011 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "sysincludes.h"
 #include "msdos.h"
 #include "stream.h"
@@ -70,14 +87,15 @@ static int recalcPreallocSize(File_t *This)
 	Fs_t *Fs = This->Fs;
 	int r;
 
+#if 0
 	if(This->FileSize & 0xc0000000) {
 		fprintf(stderr, "Bad filesize\n");
 	}
 	if(This->preallocatedSize & 0xc0000000) {
-		fprintf(stderr, "Bad preallocated size %x\n", 
+		fprintf(stderr, "Bad preallocated size %x\n",
 				(int) This->preallocatedSize);
 	}
-
+#endif
 	clus_size = Fs->cluster_size * Fs->sector_size;
 
 	currentClusters = (This->FileSize + clus_size - 1) / clus_size;
@@ -92,7 +110,7 @@ static int recalcPreallocSize(File_t *This)
 	return 0;
 }
 
-static int _loopDetect(unsigned int *oldrel, unsigned int rel, 
+static int _loopDetect(unsigned int *oldrel, unsigned int rel,
 		       unsigned int *oldabs, unsigned int absol)
 {
 	if(*oldrel && rel > *oldrel && absol == *oldabs) {
@@ -149,7 +167,7 @@ static size_t countBytes(Stream_t *Dir, unsigned int block)
 	Stream_t *Stream = GetFs(Dir);
 	DeclareThis(Fs_t);
 
-	return _countBlocks(This, block) * 
+	return _countBlocks(This, block) *
 		This->sector_size * This->cluster_size;
 }
 
@@ -195,6 +213,34 @@ void printFat(Stream_t *Stream)
 	}
 }
 
+void printFatWithOffset(Stream_t *Stream, off_t offset) {
+	File_t *This = getUnbufferedFile(Stream);
+	unsigned long n;
+	int rel;
+	off_t clusSize;
+
+	n = This->FirstAbsCluNr;
+	if(!n) {
+		printf("Root directory or empty file\n");
+		return;
+	}
+
+	clusSize = This->Fs->cluster_size * This->Fs->sector_size;
+
+	rel = 0;
+	while(offset >= clusSize) {
+		n = fatDecode(This->Fs, n);
+		rel++;
+		if(loopDetect(This, rel, n) < 0)
+			return;
+		if(n > This->Fs->last_fat)
+			return;
+		offset -= clusSize;
+	}
+
+	printf("%lu", n);
+}
+
 static int normal_map(File_t *This, off_t where, size_t *len, int mode,
 						   mt_off_t *res)
 {
@@ -254,7 +300,7 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode,
 		}
 		NewCluNr = fatDecode(This->Fs, AbsCluNr);
 		if (NewCluNr == 1 || NewCluNr == 0){
-			fprintf(stderr,"Fat problem while decoding %d %x\n", 
+			fprintf(stderr,"Fat problem while decoding %d %x\n",
 				AbsCluNr, NewCluNr);
 			exit(1);
 		}
@@ -288,8 +334,8 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode,
 	maximize(*len, (1 + CurCluNr - RelCluNr) * clus_size - offset);
 	
 	end = where + *len;
-	if(batchmode && 
-	   mode == MT_WRITE && 
+	if(batchmode &&
+	   mode == MT_WRITE &&
 	   end >= This->FileSize) {
 		*len += ROUND_UP(end, clus_size) - end;
 	}
@@ -300,7 +346,7 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode,
 		exit(1);
 	}
 
-	*res = sectorsToBytes((Stream_t*)Fs, 
+	*res = sectorsToBytes((Stream_t*)Fs,
 						  (This->PreviousAbsCluNr-2) * Fs->cluster_size +
 						  Fs->clus_start) + offset;
 	return 1;
@@ -327,7 +373,7 @@ static int root_map(File_t *This, off_t where, size_t *len, int mode,
 }
 	
 
-static int read_file(Stream_t *Stream, char *buf, mt_off_t iwhere, 
+static int read_file(Stream_t *Stream, char *buf, mt_off_t iwhere,
 					 size_t len)
 {
 	DeclareThis(File_t);
@@ -363,7 +409,7 @@ static int write_file(Stream_t *Stream, char *buf, mt_off_t iwhere, size_t len)
 		ret = WRITES(Disk, buf, pos, len);
 	if(ret > (signed int) requestedLen)
 		ret = requestedLen;
-	if (ret > 0 && 
+	if (ret > 0 &&
 	    where + ret > (off_t) This->FileSize )
 		This->FileSize = where + ret;
 	recalcPreallocSize(This);
@@ -427,9 +473,10 @@ static __inline__ time_t conv_stamp(struct directory *dir)
 	/* correct for Daylight Saving Time */
 	tmp = accum;
 	tmbuf = localtime(&tmp);
-	dst = (tmbuf->tm_isdst) ? (-60L * 60L) : 0L;
-	accum += dst;
-	
+	if(tmbuf) {
+		dst = (tmbuf->tm_isdst) ? (-60L * 60L) : 0L;
+		accum += dst;
+	}
 	return accum;
 }
 
@@ -455,7 +502,7 @@ static int free_file(Stream_t *Stream)
 {
 	DeclareThis(File_t);
 	Fs_t *Fs = This->Fs;
-	fsPreallocateClusters(Fs, -This->preallocatedClusters);       
+	fsPreallocateClusters(Fs, -This->preallocatedClusters);
 	FREE(&This->direntry.Dir);
 	freeDirCache(Stream);
 	return hash_remove(filehash, (void *) Stream, This->hint);
@@ -495,13 +542,14 @@ static int pre_allocate_file(Stream_t *Stream, mt_size_t isize)
 }
 
 static Class_t FileClass = {
-	read_file, 
-	write_file, 
+	read_file,
+	write_file,
 	flush_file, /* flush */
 	free_file, /* free */
 	0, /* get_geom */
 	get_file_data,
-	pre_allocate_file
+	pre_allocate_file,
+	get_dosConvert_pass_through
 };
 
 static unsigned int getAbsCluNr(File_t *This)
@@ -548,7 +596,7 @@ static void init_hash(void)
 }
 
 
-static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first, 
+static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first,
 				   size_t size, direntry_t *entry)
 {
 	Stream_t *Stream = GetFs(Dir);
@@ -571,7 +619,7 @@ static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first,
 		Pattern.FirstAbsCluNr = first;
 		Pattern.loopDetectRel = 0;
 		Pattern.loopDetectAbs = first;
-		if(!hash_lookup(filehash, (T_HashTableEl) &Pattern, 
+		if(!hash_lookup(filehash, (T_HashTableEl) &Pattern,
 				(T_HashTableEl **)&File, 0)){
 			File->refs++;
 			This->refs--;
@@ -628,7 +676,7 @@ Stream_t *OpenRoot(Stream_t *Dir)
 	/* make the directory entry */
 	entry.entry = -3;
 	entry.name[0] = '\0';
-	mk_entry("/", ATTR_DIR, num, 0, 0, &entry.dir);
+	mk_entry_from_base("/", ATTR_DIR, num, 0, 0, &entry.dir);
 
 	if(num)
 		size = countBytes(Dir, num);
@@ -654,7 +702,7 @@ Stream_t *OpenFileByDirentry(direntry_t *entry)
 		return OpenRoot(entry->Dir);
 	if (IS_DIR(entry))
 		size = countBytes(entry->Dir, first);
-	else 
+	else
 		size = FILE_SIZE(&entry->dir);
 	file = _internalFileOpen(entry->Dir, first, size, entry);
 	if(IS_DIR(entry)) {
