@@ -1,4 +1,20 @@
-/*
+/*  Copyright 1999 Peter Schlaile.
+ *  Copyright 1999-2002,2005-2007,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * IO to the floppyd daemon running on the local X-Server Host
  *
  * written by:
@@ -24,13 +40,16 @@
 
 typedef unsigned char Byte;
 typedef unsigned long Dword;
+typedef mt_off_t Qword;
 
 const char* AuthErrors[] = {
-	"Auth success!",
-	"Auth failed: Packet oversized!",
-	"Auth failed: X-Cookie doesn't match!",
-	"Auth failed: Wrong transmission protocol version!",
-	"Auth failed: Device locked!"
+	"Auth success",
+	"Auth failed: Packet oversized",
+	"Auth failed: X-Cookie doesn't match",
+	"Auth failed: Wrong transmission protocol version",
+	"Auth failed: Device locked"
+	"Auth failed: Bad packet",
+	"Auth failed: I/O Error"
 };
 
 
@@ -77,7 +96,8 @@ static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *displa
 	/* Version negotiation */
 	dword2byte(4,buf);
 	dword2byte(floppyd->version,buf+4);
-	write(sock, buf, 8);
+	if(write(sock, buf, 8) < 8)
+		return AUTH_IO_ERROR;
 
 	if ( (l = read_dword(sock)) < 4) {
 		return AUTH_WRONGVERSION;
@@ -95,7 +115,8 @@ static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *displa
 		floppyd->capabilities = read_dword(sock);
 
 	dword2byte(filelen, (Byte *)xcookie);
-	write(sock, xcookie, filelen+4);
+	if(write(sock, xcookie, filelen+4) < filelen + 4)
+		return AUTH_IO_ERROR;
 
 	if (read_dword(sock) != 4) {
 		return AUTH_PACKETOVERSIZE;
@@ -119,7 +140,8 @@ static int floppyd_reader(int fd, char* buffer, int len)
 	buf[4] = OP_READ;
 	dword2byte(4, buf+5);
 	dword2byte(len, buf+9);
-	write(fd, buf, 13);
+	if(write(fd, buf, 13) < 13)
+		return AUTH_IO_ERROR;
 
 	if (read_dword(fd) != 8) {
 		errno = EIO;
@@ -157,8 +179,10 @@ static int floppyd_writer(int fd, char* buffer, int len)
 	buf[4] = OP_WRITE;
 	dword2byte(len, buf+5);
 
-	write(fd, buf, 9);
-        write(fd, buffer, len);
+	if(write(fd, buf, 9) < 9)
+		return AUTH_IO_ERROR;
+	if(write(fd, buffer, len) < len)
+		return AUTH_IO_ERROR;
 	
 	if (read_dword(fd) != 8) {
 		errno = EIO;
@@ -191,7 +215,8 @@ static int floppyd_lseek(int fd, mt_off_t offset, int whence)
 	dword2byte(truncBytes32(offset), buf+9);
 	dword2byte(whence, buf+13);
 	
-	write(fd, buf, 17);
+	if(write(fd, buf, 17) < 17)
+		return AUTH_IO_ERROR;
        
 	if (read_dword(fd) != 8) {
 		errno = EIO;
@@ -206,6 +231,35 @@ static int floppyd_lseek(int fd, mt_off_t offset, int whence)
 	return gotlen;
 }
 
+static mt_off_t floppyd_lseek64(int fd, mt_off_t offset, int whence) 
+{
+	Dword errcode;
+	Qword gotlen;
+	Byte buf[32];
+	
+	dword2byte(1, buf);
+	buf[4] = OP_SEEK64;
+	
+	dword2byte(12, buf+5);
+	qword2byte(offset, buf+9);
+	dword2byte(whence, buf+17);
+	
+	if(write(fd, buf, 21) < 21)
+		return AUTH_IO_ERROR;
+       
+	if (read_dword(fd) != 12) {
+		errno = EIO;
+		return -1;
+	}
+
+	gotlen = read_qword(fd);
+	errcode = read_dword(fd);
+
+	errno = errcode;
+	
+	return gotlen;
+}
+
 static int floppyd_open(RemoteFile_t *This, int mode) 
 {
 	Dword errcode;
@@ -225,7 +279,8 @@ static int floppyd_open(RemoteFile_t *This, int mode)
 	dword2byte(4, buf+5);
 	dword2byte(This->drive, buf+9);
 
-	write(This->fd, buf, 13);
+	if(write(This->fd, buf, 13) < 13)
+		return AUTH_IO_ERROR;
        
 	if (read_dword(This->fd) != 8) {
 		errno = EIO;
@@ -254,10 +309,18 @@ static int floppyd_io(Stream_t *Stream, char *buf, mt_off_t where, int len,
 	where += This->offset;
 
 	if (where != This->lastwhere ){
-		if(floppyd_lseek( This->fd, where, SEEK_SET) < 0 ){
-			perror("floppyd_lseek");
-			This->lastwhere = (mt_off_t) -1;
-			return -1;
+		if(This->capabilities & FLOPPYD_CAP_LARGE_SEEK) {
+			if(floppyd_lseek64( This->fd, where, SEEK_SET) < 0 ){
+				perror("floppyd_lseek64");
+				This->lastwhere = (mt_off_t) -1;
+				return -1;
+			}
+		} else {
+			if(floppyd_lseek( This->fd, where, SEEK_SET) < 0 ){
+				perror("floppyd_lseek");
+				This->lastwhere = (mt_off_t) -1;
+				return -1;
+			}
 		}
 	}
 	ret = io(This->fd, buf, len);
@@ -291,7 +354,8 @@ static int floppyd_flush(Stream_t *Stream)
 	dword2byte(1, buf+5);
 	buf[9] = '\0';
 
-	write(This->fd, buf, 10);
+	if(write(This->fd, buf, 10) < 10)
+		return AUTH_IO_ERROR;
 
 	if (read_dword(This->fd) != 8) {
 		errno = EIO;
@@ -313,7 +377,8 @@ static int floppyd_free(Stream_t *Stream)
 	if (This->fd > 2) {
 		dword2byte(1, buf);
 		buf[4] = OP_CLOSE;
-		write(This->fd, buf, 5);
+		if(write(This->fd, buf, 5) < 5)
+			return AUTH_IO_ERROR;
 		shutdown(This->fd, 1);
 		if (read_dword(This->fd) != 8) {
 		    errno = EIO;
@@ -334,7 +399,7 @@ static int floppyd_free(Stream_t *Stream)
 
 static int floppyd_geom(Stream_t *Stream, struct device *dev, 
 		     struct device *orig_dev,
-		     int media, struct bootsector *boot)
+		     int media, union bootsector *boot)
 {
 	size_t tot_sectors;
 	int sect_per_track;
@@ -489,11 +554,7 @@ static int connect_to_server(IPaddr_t ip, short port)
 	 */
 	
 	addr.sin_family = AF_INET;
-#ifndef HAVE_HTONS
-	addr.sin_port = myhtons(port);
-#else	
 	addr.sin_port = htons(port);
-#endif	
 	addr.sin_addr.s_addr = ip;
 	
         /*
@@ -520,7 +581,7 @@ static int ConnectToFloppyd(RemoteFile_t *floppyd, const char* name,
 
 Stream_t *FloppydOpen(struct device *dev, struct device *dev2,
 		      char *name, int mode, char *errmsg,
-		      int mode2, int locked)
+		      int mode2, int locked, mt_size_t *maxSize)
 {
 	RemoteFile_t *This;
 
@@ -553,6 +614,11 @@ Stream_t *FloppydOpen(struct device *dev, struct device *dev2,
 		return NULL;
 	}
 
+	if(maxSize) {
+		*maxSize = 
+			(This->capabilities & FLOPPYD_CAP_LARGE_SEEK) ?
+			max_off_t_seek : max_off_t_31;
+	}
 	return (Stream_t *) This;
 }
 
