@@ -1,6 +1,6 @@
 #ifndef MTOOLS_MTOOLS_H
 #define MTOOLS_MTOOLS_H
-/*  Copyright 1996-2005,2007-2009 Alain Knaff.
+/*  Copyright 1996-2005,2007-2011 Alain Knaff.
  *  This file is part of mtools.
  *
  *  Mtools is free software: you can redistribute it and/or modify
@@ -189,6 +189,7 @@ extern int batchmode;
 char get_default_drive(void);
 void set_cmd_line_image(char *img, int flags);
 void read_config(void);
+off_t str_to_offset(char *str);
 extern struct device *devices;
 extern struct device const_devices[];
 extern const int nr_const_devices;
@@ -215,6 +216,7 @@ void mmd(int argc, char **argv, int type);
 void mmount(int argc, char **argv, int type);
 void mmove(int argc, char **argv, int type);
 void mpartition(int argc, char **argv, int type);
+void mshortname(int argc, char **argv, int mtype);
 void mshowfat(int argc, char **argv, int mtype);
 void mtoolstest(int argc, char **argv, int type);
 void mzip(int argc, char **argv, int type);
